@@ -1,7 +1,7 @@
 /* modpriv.h: Stuff needed by both modules.c and modcall.c, but should not be
  * accessed from anywhere else.
  *
- * Version: $Id: modpriv.h,v 1.1 2000/12/21 06:31:21 pacman Exp $ */
+ * Version: $Id: modpriv.h,v 1.3 2004/02/26 19:04:19 aland Exp $ */
 #include "radiusd.h"
 #include "modules.h"
 #include "ltdl.h"
@@ -18,7 +18,7 @@ typedef struct module_list_t {
 
 /*
  *	Per-instance data structure, to correlate the modules
- *	with the instance names (may NOT be the module names!), 
+ *	with the instance names (may NOT be the module names!),
  *	and the per-instance data structures.
  */
 typedef struct module_instance_t {
@@ -26,7 +26,7 @@ typedef struct module_instance_t {
 	char			name[MAX_STRING_LEN];
 	module_list_t		*entry;
 	void                    *insthandle;
-#if HAVE_PTHREAD_H
+#ifdef HAVE_PTHREAD_H
 	pthread_mutex_t		*mutex;
 #endif
 } module_instance_t;
