@@ -1,7 +1,7 @@
 /*
  * files.c	Read config files into memory.
  *
- * Version:     $Id: files.c,v 1.71 2002/10/29 22:55:26 aland Exp $
+ * Version:     $Id: files.c,v 1.85 2004/04/06 20:43:49 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,14 +22,14 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: files.c,v 1.71 2002/10/29 22:55:26 aland Exp $";
+static const char rcsid[] = "$Id: files.c,v 1.85 2004/04/06 20:43:49 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
 
 #include <sys/stat.h>
 
-#if HAVE_NETINET_IN_H
+#ifdef HAVE_NETINET_IN_H
 #	include <netinet/in.h>
 #endif
 
@@ -61,59 +61,6 @@ void pairlist_free(PAIR_LIST **pl)
 }
 
 
-/*
- *	Fixup a check line.
- *	If User-Password or Crypt-Password is set, but there is no
- *	Auth-Type, add one (kludge!).
- */
-static void auth_type_fixup(VALUE_PAIR **check)
-{
-	VALUE_PAIR *vp;
-	VALUE_PAIR *c = NULL;
-	int n = 0;
-
-	/*
-	 *	See if a password is present. Return right away
-	 *	if we see Auth-Type.
-	 */
-	for (vp = *check; vp; vp = vp->next) {
-		if (vp->attribute == PW_AUTHTYPE)
-			return;
-		if (vp->attribute == PW_PASSWORD) {
-			c = vp;
-			n = PW_AUTHTYPE_LOCAL;
-		}
-		if (vp->attribute == PW_CRYPT_PASSWORD) {
-			c = vp;
-			n = PW_AUTHTYPE_CRYPT;
-		}
-	}
-
-	if (c == NULL)
-		return;
-
-	/*
-	 *	Add an Auth-Type attribute.
-	 *	
-	 */
-	if ((vp = paircreate(PW_AUTHTYPE, PW_TYPE_INTEGER)) == NULL) {
-		radlog(L_CONS|L_ERR, "no memory");
-		exit(1);
-	}
-	vp->lvalue = n;
-	vp->operator = T_OP_ADD;
-	strcpy(vp->strvalue, "Local");
-
-	vp->next = *check;
-	*check = vp;
-
-	for(vp = *check; vp; vp = vp->next) {
-		DEBUG2("  auth_type_fixup: %s [%d]", vp->name, vp->attribute);
-	}
-
-}
-
-
 #define FIND_MODE_NAME  0
 #define FIND_MODE_REPLY 1
 
@@ -126,11 +73,12 @@ int pairlist_read(const char *file, PAIR_LIST **list, int complain)
 	FILE *fp;
 	int mode = FIND_MODE_NAME;
 	char entry[256];
-	char buffer[256];
+	char buffer[8192];
 	char *ptr, *s;
 	VALUE_PAIR *check_tmp;
 	VALUE_PAIR *reply_tmp;
-	PAIR_LIST *pl = NULL, *last = NULL, *t;
+	PAIR_LIST *pl = NULL, *t;
+	PAIR_LIST **last = &pl;
 	int lineno = 0;
 	int old_lineno = 0;
 	LRAD_TOKEN parsecode;
@@ -141,7 +89,7 @@ int pairlist_read(const char *file, PAIR_LIST **list, int complain)
 	 *	more useful...
 	 */
 	if ((fp = fopen(file, "r")) == NULL) {
-		if (!complain) 
+		if (!complain)
 			return -1;
 		radlog(L_CONS|L_ERR, "Couldn't open %s for reading: %s",
 				file, strerror(errno));
@@ -160,6 +108,20 @@ int pairlist_read(const char *file, PAIR_LIST **list, int complain)
 			return -1;
 		}
 		if (buffer[0] == '#' || buffer[0] == '\n') continue;
+
+		/*
+		 *	If the line contains nothing but whitespace,
+		 *	ignore it.
+		 */
+		ptr = buffer;
+		while ((ptr[0] == ' ') ||
+		       (ptr[0] == '\t') ||
+		       (ptr[0] == '\r') ||
+		       (ptr[0] == '\n')) {
+			ptr++;
+		}
+		if (ptr[0] == '\0') continue;
+
 parse_again:
 		if(mode == FIND_MODE_NAME) {
 			/*
@@ -168,8 +130,8 @@ parse_again:
 			if (isspace((int) buffer[0]))  {
 				if (parsecode != T_EOL) {
 					radlog(L_ERR|L_CONS,
-							"%s[%d]: Unexpected trailing comma for entry %s",
-							file, lineno, entry);
+					       "%s[%d]: Unexpected trailing comma for entry %s",
+					       file, lineno, entry);
 					fclose(fp);
 					return -1;
 				}
@@ -211,18 +173,21 @@ parse_again:
 				if (pairlist_read(s, &t, 0) != 0) {
 					pairlist_free(&pl);
 					radlog(L_ERR|L_CONS,
-							"%s[%d]: Could not open included file %s: %s",
-							file, lineno, s, strerror(errno));
+					       "%s[%d]: Could not open included file %s: %s",
+					       file, lineno, s, strerror(errno));
 					fclose(fp);
 				return -1;
 				}
-				if (last)
-					last->next = t;
-				else
-					pl = t;
-				last = t;
-				while (last && last->next)
-					last = last->next;
+				*last = t;
+
+				/*
+				 *	t may be NULL, it may have one
+				 *	entry, or it may be a linked list
+				 *	of entries.  Go to the end of the
+				 *	list.
+				 */
+				while (*last)
+					last = &((*last)->next);
 				continue;
 			}
 
@@ -242,8 +207,8 @@ parse_again:
 				return -1;
 			} else if (parsecode == T_COMMA) {
 				radlog(L_ERR|L_CONS,
-						"%s[%d]: Unexpected trailing comma in check item list for entry %s",
-						file, lineno, entry);
+				       "%s[%d]: Unexpected trailing comma in check item list for entry %s",
+				       file, lineno, entry);
 				fclose(fp);
 				return -1;
 			}
@@ -254,8 +219,8 @@ parse_again:
 			if(*buffer == ' ' || *buffer == '\t') {
 				if (parsecode != T_COMMA) {
 					radlog(L_ERR|L_CONS,
-							"%s[%d]: Syntax error: Previous line is missing a trailing comma for entry %s",
-							file, lineno, entry);
+					       "%s[%d]: Syntax error: Previous line is missing a trailing comma for entry %s",
+					       file, lineno, entry);
 					fclose(fp);
 					return -1;
 				}
@@ -268,8 +233,8 @@ parse_again:
 				if (parsecode < 1) {
 					pairlist_free(&pl);
 					radlog(L_ERR|L_CONS,
-							"%s[%d]: Parse error (reply) for entry %s: %s",
-							file, lineno, entry, librad_errstr);
+					       "%s[%d]: Parse error (reply) for entry %s: %s",
+					       file, lineno, entry, librad_errstr);
 					fclose(fp);
 					return -1;
 				}
@@ -280,7 +245,6 @@ parse_again:
 				 */
 				t = rad_malloc(sizeof(PAIR_LIST));
 
-				auth_type_fixup(&check_tmp);
 				memset(t, 0, sizeof(*t));
 				t->name = strdup(entry);
 				t->check = check_tmp;
@@ -288,11 +252,9 @@ parse_again:
 				t->lineno = old_lineno;
 				check_tmp = NULL;
 				reply_tmp = NULL;
-				if (last)
-					last->next = t;
-				else
-					pl = t;
-				last = t;
+
+				*last = t;
+				last = &(t->next);
 
 				mode = FIND_MODE_NAME;
 				if (buffer[0] != 0)
@@ -370,6 +332,7 @@ int read_realms_file(const char *file)
 	char *s, *p;
 	int lineno = 0;
 	REALM *c, **tail;
+	int got_realm = FALSE;
 
 	realm_free(mainconfig.realms);
 	mainconfig.realms = NULL;
@@ -382,6 +345,7 @@ int read_realms_file(const char *file)
 		   So this is a non-fatal error.  */
 		return 0;
 	}
+
 	while(fgets(buffer, 256, fp) != NULL) {
 		lineno++;
 		if (!feof(fp) && (strchr(buffer, '\n') == NULL)) {
@@ -397,6 +361,7 @@ int read_realms_file(const char *file)
 			continue;
 		}
 
+		got_realm = TRUE;
 		c = rad_malloc(sizeof(REALM));
 		memset(c, 0, sizeof(REALM));
 
@@ -416,8 +381,8 @@ int read_realms_file(const char *file)
 			 */
 			c->acct_ipaddr = c->ipaddr = htonl(INADDR_NONE);
 			c->secret[0] = '\0';
-			c->auth_port = auth_port;
-			c->acct_port = acct_port;
+			c->auth_port = 0;
+			c->acct_port = 0;
 
 		} else {
 			RADCLIENT *client;
@@ -448,14 +413,16 @@ int read_realms_file(const char *file)
 		 */
 		if (strlen(hostnm) >= sizeof(c->server)) {
 			radlog(L_ERR, "%s[%d]: server name of length %d is greater than the allowed maximum of %d.",
-					file, lineno,
-					strlen(hostnm), sizeof(c->server) - 1);
+			       file, lineno,
+			       (int) strlen(hostnm),
+			       (int) sizeof(c->server) - 1);
 			return -1;
 		}
 		if (strlen(realm) > sizeof(c->realm)) {
 			radlog(L_ERR, "%s[%d]: realm of length %d is greater than the allowed maximum of %d.",
-					file, lineno,
-					strlen(realm), sizeof(c->realm) - 1);
+			       file, lineno,
+			       (int) strlen(realm),
+			       (int) sizeof(c->realm) - 1);
 			return -1;
 		}
 
@@ -487,6 +454,13 @@ int read_realms_file(const char *file)
 	}
 	fclose(fp);
 
+	/*
+	 *	Complain only if the realms file has content.
+	 */
+	if (got_realm) {
+		radlog(L_INFO, "Using deprecated realms file.  Support for this will go away soon.");
+	}
+
 	return 0;
 }
 #endif /* BUILDDBM */
@@ -500,33 +474,47 @@ void realm_disable(uint32_t ipaddr, int port)
 	time_t now;
 
 	now = time(NULL);
-	for(cl = mainconfig.realms; cl; cl = cl->next)
+	for(cl = mainconfig.realms; cl; cl = cl->next) {
 		if ((ipaddr == cl->ipaddr) && (port == cl->auth_port)) {
+			/*
+			 *	If we've received a reply (any reply)
+			 *	from the home server in the time spent
+			 *	re-sending this request, then don't mark
+			 *	the realm as dead.
+			 */
+			if (cl->last_reply > (( now - mainconfig.proxy_retry_delay * mainconfig.proxy_retry_count ))) {
+				continue;
+			}
+
 			cl->active = FALSE;
 			cl->wakeup = now + mainconfig.proxy_dead_time;
 			radlog(L_PROXY, "marking authentication server %s:%d for realm %s dead",
 				cl->server, port, cl->realm);
 		} else if ((ipaddr == cl->acct_ipaddr) && (port == cl->acct_port)) {
+			if (cl->last_reply > (( now - mainconfig.proxy_retry_delay * mainconfig.proxy_retry_count ))) {
+				continue;
+			}
+
 			cl->acct_active = FALSE;
 			cl->acct_wakeup = now + mainconfig.proxy_dead_time;
 			radlog(L_PROXY, "marking accounting server %s:%d for realm %s dead",
-				cl->server, port, cl->realm);
+				cl->acct_server, port, cl->realm);
 		}
+	}
 }
 
 /*
  *	Find a realm in the REALM list.
  */
-REALM *realm_find(const char *realm, int acct)
+REALM *realm_find(const char *realm, int accounting)
 {
-	REALM *cl, *realmptr;
+	REALM *cl;
 	REALM *default_realm = NULL;
-	REALM *rr_array[maximum_proxies];
 	time_t now;
-	int dead_match = 0, efform = 0, p = 0, i = 0;
-	static int proxy_return;
+	int dead_match = 0;
 
 	now = time(NULL);
+
 	/*
 	 *	If we're passed a NULL realm pointer,
 	 *	then look for a "NULL" realm string.
@@ -534,7 +522,7 @@ REALM *realm_find(const char *realm, int acct)
 	if (realm == NULL) {
 		realm = "NULL";
 	}
-	
+
 	for (cl = mainconfig.realms; cl; cl = cl->next) {
 		/*
 		 *	Wake up any sleeping realm.
@@ -550,8 +538,8 @@ REALM *realm_find(const char *realm, int acct)
 		 *	Asked for auth/acct, and the auth/acct server
 		 *	is not active.  Skip it.
 		 */
-		if ((!acct && !cl->active) ||
-		    (acct && !cl->acct_active)) {
+		if ((!accounting && !cl->active) ||
+		    (accounting && !cl->acct_active)) {
 
 			/*
 			 *	We've been asked to NOT fall through
@@ -568,108 +556,14 @@ REALM *realm_find(const char *realm, int acct)
 
 		/*
 		 *	If it matches exactly, return it.
+		 *
+		 *	Note that we just want ONE live realm
+		 *	here.  We don't care about round-robin, or
+		 *	scatter techniques, as that's more properly
+		 *	the responsibility of the proxying code.
 		 */
 		if (strcasecmp(cl->realm, realm) == 0) {
-                        /*
-                         *    Seperate parsing of list for round-robin
-                         */
-			if(cl->total > 1 && cl->ldflag == 1) {
-				/*Get all of the realms from initial list*/
-				for(i = 1; i <= cl->total; i++) {
-					for(realmptr = mainconfig.realms; realmptr; 
-                                            realmptr = realmptr->next) {
-						if((strcasecmp(realmptr->realm,
-                                                   cl->realm)) == 0 &&
-						   (realmptr->node == i)) {
-							rr_array[i] = realmptr;
-							if(realmptr->active == TRUE)
-								p++;
-							break;
-						}
-						else {
-							continue;
-						}
-					}
-				}
-				/*if only one realm active, return it*/
-				if(p == 1) {
-					for (p = 1; p < i; p++) {
-						realmptr = rr_array[p];
-						if(realmptr->active == TRUE) {
-							return realmptr;
-						}
-					}
-				}
-				for(p = 1; p < i; p++) {
-					realmptr = rr_array[p];
-					if(realmptr->chose == 1) {
-						efform++;
-					}
-				}
-
-				/*    
-                                 *    if none available, 
-				 *    and last realm has
-				 *    been returned as access-reply
-                                 */
-
-				if(efform == cl->total && proxy_return == 0) {
-					for(p = 1; p < i; p++) {
-						realmptr = rr_array[p];
-						realmptr->chose = 0;
-					}
-				}
-
-				/*    
-                                 *    if this realm hasn't been chosen and 
-				 *    this is not being returned for
-                                 *    access reply
-				 *    else if the realm has been chosen 
-                                 *    and this is for access-reply
-                                 *    else this realm is dead (reset if
-				 *    this is the last realm in the list)
-				 */
-
-                                for(p = 1; p < i; p++) {
-					realmptr = rr_array[p];
-					if((realmptr->chose == 0) &&
-                                           (proxy_return < realmptr->node) &&
-					   (realmptr->active == TRUE && realmptr->acct_active == TRUE)
-                                        ) {
-						proxy_return = realmptr->node;
-						realmptr->chose = 1;
-						return realmptr;
-					}
-					else if ((realmptr->chose == 1) &&
-						 (proxy_return == realmptr->node)
-                                        ) {
-						if(proxy_return + 1 > realmptr->total) {
-							proxy_return = 0;
-						}
-						realmptr->chose = 0;
-						return realmptr;
-					}
-					else 
-					{
-						if((realmptr->active == FALSE ||
-						    realmptr->acct_active == FALSE ) &&
-						   (realmptr->node == realmptr->total)
-                                                ) {
-							proxy_return = 0;
-						}
-						continue;
-					}
-				}
-			}
-			
-			/*    
-			 *    else we have single 
-                         *    realm or non-rr realm
-                         */
-			
-			else {
-				return cl;
-			}
+			return cl;
 		}
 
 		/*
@@ -688,7 +582,33 @@ REALM *realm_find(const char *realm, int acct)
 	 *	we return NULL, which means "no match found".
 	 */
 	if (!mainconfig.proxy_fallback && dead_match) {
-		return NULL;
+		if (mainconfig.wake_all_if_all_dead) {
+			REALM *rcl = NULL;
+			for (cl = mainconfig.realms; cl; cl = cl->next) {
+				if(strcasecmp(cl->realm,realm) == 0) {
+					if (!accounting && !cl->active) {
+						cl->active = TRUE;
+						rcl = cl;
+					}
+					else if (accounting &&
+						 !cl->acct_active) {
+						cl->acct_active = TRUE;
+						rcl = cl;
+					}
+				}
+			}
+			return rcl;
+		}
+		else {
+			return NULL;
+		}
+	}
+
+	/*      If we didn't find the realm 'NULL' don't return the
+	 *      DEFAULT entry.
+	 */
+	if ((strcmp(realm, "NULL")) == 0) {
+	  return NULL;
 	}
 
 	/*
@@ -703,6 +623,8 @@ REALM *realm_find(const char *realm, int acct)
 
 /*
  *	Find a realm for a proxy reply by proxy's IP
+ *
+ *	Note that we don't do anything else.
  */
 REALM *realm_findbyaddr(uint32_t ipaddr, int port)
 {
@@ -711,54 +633,20 @@ REALM *realm_findbyaddr(uint32_t ipaddr, int port)
 	/*
 	 *	Note that we do NOT check for inactive realms!
 	 *
-	 *	If we get a packet from an end server, then we mark it
-	 *	as active, and return the realm.
+	 *	The purpose of this code is simply to find a matching
+	 *	source IP/Port pair, for a home server which is allowed
+	 *	to send us proxy replies.  If we get a reply, then it
+	 *	doesn't matter if we think the realm is inactive.
 	 */
-	for(cl = mainconfig.realms; cl != NULL; cl = cl->next)
+	for (cl = mainconfig.realms; cl != NULL; cl = cl->next) {
 		if ((ipaddr == cl->ipaddr) && (port == cl->auth_port)) {
-			cl->active = TRUE;
 			return cl;
+
 		} else if ((ipaddr == cl->acct_ipaddr) && (port == cl->acct_port)) {
-			cl->acct_active = TRUE;
 			return cl;
 		}
+	}
 
 	return NULL;
 }
 
-/*
- *    Catch any errors in set-up of RR list or exceeding max_proxies
- */
-void check_proxies(int max_config) {
-	REALM *next, *rptr;
-	if(max_config > 0) {
-		maximum_proxies = max_config;
-		for(rptr=mainconfig.realms; rptr; rptr = rptr->next) {
-			if(rptr->total > max_config || rptr->total >= 1) {
-				for(next = mainconfig.realms; next; next = next->next) {
-					if(next->ldflag != 0 &&
-                                           next->ldflag != 1) {
-						radlog(L_ERR,
-						    "Invalid load balance flag on realm: %s entry %d\n",
-                                                     next->realm, next->node);
-						exit(1);
-					}
-					else if((strcasecmp(rptr->realm, next->realm) == 0) && 
-                                            rptr->ldflag != next->ldflag){ 
-                                               radlog(L_ERR,
-                                                   "Inconsistent ldflag for realm: %s entries %d and %d.\n", 
-                                                    rptr->realm, rptr->node, next->node);
-					       exit(1);
-					}
-				        else if (next->total > max_config) {
-				            radlog(L_ERR,
-				                "Too many entries (%d) for realm: %s. Max is %d.\n",
- 				 	         next->total, next->realm, max_config);
-				            exit(1);
-				        }
-				}
-			}
-		}
-		maximum_proxies++;
-	}
-}
