@@ -1,7 +1,7 @@
 /*
  * radiusd.c	Main loop of the radius server.
  *
- * Version:	$Id: radiusd.c,v 1.321.2.1 2004/07/05 15:36:08 aland Exp $
+ * Version:	$Id: radiusd.c,v 1.321.2.2 2005/04/11 23:45:21 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -27,7 +27,7 @@
 
 /* don't look here for the version, run radiusd -v or look in version.c */
 static const char rcsid[] =
-"$Id: radiusd.c,v 1.321.2.1 2004/07/05 15:36:08 aland Exp $";
+"$Id: radiusd.c,v 1.321.2.2 2005/04/11 23:45:21 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -95,7 +95,6 @@ int need_reload = FALSE;
 int sig_hup_block = FALSE;
 const char *radiusd_version = "FreeRADIUS Version " RADIUSD_VERSION ", for host " HOSTINFO ", built on " __DATE__ " at " __TIME__;
 
-static int got_child = FALSE;
 static time_t time_now;
 static pid_t radius_pid;
 
@@ -103,7 +102,6 @@ static pid_t radius_pid;
  *  Configuration items.
  */
 static int dont_fork = FALSE;
-static int needs_child_cleanup = 0;
 static time_t start_time = 0;
 static int spawn_flag = TRUE;
 static int do_exit = 0;
@@ -115,9 +113,6 @@ static void usage(int);
 
 static void sig_fatal (int);
 static void sig_hup (int);
-#ifdef HAVE_PTHREAD_H
-static void sig_cleanup(int);
-#endif
 
 static int rad_status_server(REQUEST *request);
 
@@ -1075,8 +1070,6 @@ int main(int argc, char *argv[])
 	if (spawn_flag == TRUE) {
 		thread_pool_init();
 	}
-
-	rad_exec_init();
 #else
 	/*
 	 *	Without threads, we ALWAYS run in single-server mode.
@@ -1155,29 +1148,6 @@ int main(int argc, char *argv[])
 #endif
 	}
 
-#ifdef HAVE_PTHREAD_H
-	/*
-	 *	If we have pthreads, then the child threads block
-	 *	SIGCHLD, and the main server thread catches it.
-	 *
-	 *	That way, the SIGCHLD handler can grab the exit status,
-	 *	and save it for the child thread.
-	 *
-	 *	If we don't have pthreads, then each child process
-	 *	will do a waitpid(), and we ignore SIGCHLD.
-	 *
-	 *	Once we have multiple child processes to handle
-	 *	requests, and shared memory, then we've got to
-	 *	re-enable SIGCHLD catching.
-	 */
-#ifdef HAVE_SIGACTION
-	act.sa_handler = sig_cleanup;
-	sigaction(SIGCHLD, &act, NULL);
-#else
-	signal(SIGCHLD, sig_cleanup);
-#endif
-#endif
-
 	radlog(L_INFO, "Ready to process requests.");
 	start_time = time(NULL);
 
@@ -1917,56 +1887,6 @@ postpone_request:
 }
 
 
-#ifdef HAVE_PTHREAD_H
-static void sig_cleanup(int sig)
-{
-	int status;
-	pid_t pid;
-
-	sig = sig; /* -Wunused */
-
-	got_child = FALSE;
-
-	needs_child_cleanup = 0;  /* reset the queued cleanup number */
-
-	/*
-	 *  Reset the signal handler, if required.
-	 */
-	reset_signal(SIGCHLD, sig_cleanup);
-
-	/*
-	 *	Wait for the child, without hanging.
-	 */
-	for (;;) {
-		pid = waitpid((pid_t)-1, &status, WNOHANG);
-		if (pid <= 0)
-			return;
-
-		/*
-		 *  Check to see if the child did a bad thing.
-		 *  If so, kill ALL processes in the current
-		 *  process group, to prevent further attacks.
-		 */
-		if (debug_flag && (WIFSIGNALED(status))) {
-			radlog(L_ERR|L_CONS, "MASTER: Child PID %d failed to catch "
-					"signal %d: killing all active servers.\n",
-					pid, WTERMSIG(status));
-			kill(-radius_pid, SIGTERM);
-			exit(1);
-		}
-
-		/*
-		 *	If we have pthreads, then the only children
-		 *	are from Exec-Program.  We don't care about them,
-		 *	so once we've grabbed their PID's, we're done.
-		 */
-#ifdef HAVE_PTHREAD_H
-		rad_savepid(pid, status);
-#endif /* !defined HAVE_PTHREAD_H */
-	}
-}
-#endif /* HAVE_PTHREAD_H */
-
 /*
  *  Display the syntax for starting this program.
  */
