@@ -2,7 +2,7 @@
  * xlat.c	Translate strings.  This is the first version of xlat
  * 		incorporated to RADIUS
  *
- * Version:	$Id: xlat.c,v 1.72.2.2 2004/07/13 19:48:56 aland Exp $
+ * Version:	$Id: xlat.c,v 1.72.2.3 2004/09/09 14:31:07 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 static const char rcsid[] =
-"$Id: xlat.c,v 1.72.2.2 2004/07/13 19:48:56 aland Exp $";
+"$Id: xlat.c,v 1.72.2.3 2004/09/09 14:31:07 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -56,6 +56,10 @@ static const char *internal_xlat[] = {"check",
 				      "proxy-request",
 				      "proxy-reply",
 				      NULL};
+
+#if REQUEST_MAX_REGEX > 8
+#error Please fix the following line
+#endif
 static int xlat_inst[] = { 0, 1, 2, 3, 4, 5, 6, 7, 8 };	/* up to 8 for regex */
 
 
@@ -297,7 +301,7 @@ int xlat_register(const char *module, RAD_XLAT_FUNC func, void *instance)
 		 *	Register xlat's for regexes.
 		 */
 		buffer[1] = '\0';
-		for (i = 0; i <= 8; i++) {
+		for (i = 0; i <= REQUEST_MAX_REGEX; i++) {
 			buffer[0] = '0' + i;
 			xlat_register(buffer, xlat_regex, &xlat_inst[i]);
 			c = xlat_find(buffer);
