@@ -1,7 +1,7 @@
 /*
  * rlm_eap_ttls.c  contains the interfaces that are called from eap
  *
- * Version:     $Id: ttls.c,v 1.17 2004/04/19 20:21:19 aland Exp $
+ * Version:     $Id: ttls.c,v 1.17.2.2 2005/06/03 17:13:58 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -639,6 +639,7 @@ static int process_reply(EAP_HANDLER *handler, tls_session_t *tls_session,
 			 *	tunneled user!
 			 */
 			if (t->use_tunneled_reply) {
+				pairdelete(&reply->vps, PW_PROXY_STATE);
 				pairadd(&request->reply->vps, reply->vps);
 				reply->vps = NULL;
 			}
@@ -710,7 +711,7 @@ static int process_reply(EAP_HANDLER *handler, tls_session_t *tls_session,
 
 	default:
 		DEBUG2("  TTLS: Unknown RADIUS packet type %d: rejecting tunneled user", reply->code);
-		rcode = RLM_MODULE_REJECT;
+		rcode = RLM_MODULE_INVALID;
 		break;
 	}
 
@@ -817,7 +818,7 @@ static int eapttls_postproxy(EAP_HANDLER *handler, void *data)
 	switch (rcode) {
 	case RLM_MODULE_REJECT:
 		DEBUG2("  TTLS: Reply was rejected");
-		return 0;
+		break;
 
 	case RLM_MODULE_HANDLED:
 		DEBUG2("  TTLS: Reply was handled");
