@@ -4,7 +4,7 @@
  *		The code in here was borrowed from the cache.c module
  *		and adapted to be a standalone set of functions.
  *
- * Version: $Id: compat.c,v 1.1.6.1 2003/09/12 19:18:04 phampson Exp $
+ * Version: $Id: compat.c,v 1.4 2004/02/26 19:04:37 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -21,8 +21,8 @@
  *   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  * Copyright 2001  The FreeRADIUS server project.
- */    
-static const char rcsid[] = "$Id: compat.c,v 1.1.6.1 2003/09/12 19:18:04 phampson Exp $";
+ */
+static const char rcsid[] = "$Id: compat.c,v 1.4 2004/02/26 19:04:37 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -36,7 +36,7 @@ static const char rcsid[] = "$Id: compat.c,v 1.1.6.1 2003/09/12 19:18:04 phampso
 
 #include "config.h"
 
-#if HAVE_SHADOW_H
+#ifdef HAVE_SHADOW_H
 #  include <shadow.h>
 #endif
 
@@ -92,7 +92,7 @@ struct passwd *rad_fgetpwent(FILE *pwhandle) {
 	ptr = buffer;
 	RAD_EXTRACT_FIELD("Username", username);
 	pwbuf.pw_name = username;
-	
+
 	/* Get (encrypted) password from password file (shadow comes later) */
 	if (*ptr != '\0') ptr++;
 	RAD_EXTRACT_FIELD("Password", userpwd);
@@ -102,12 +102,12 @@ struct passwd *rad_fgetpwent(FILE *pwhandle) {
 	if (*ptr != '\0') ptr++;
 	RAD_EXTRACT_FIELD("UID", uidtmp);
 	pwbuf.pw_uid = atoi(uidtmp);
-	
+
 	/* Get gid from the password file */
 	if (*ptr != '\0') ptr++;
 	RAD_EXTRACT_FIELD("GID", gidtmp);
 	pwbuf.pw_gid = atoi(gidtmp);
-	
+
 	/* Get the GECOS (name) field from the password file */
 	if (*ptr != '\0') ptr++;
 	RAD_EXTRACT_FIELD("GECOS", gecostmp);
@@ -176,7 +176,7 @@ shadow_pwd_t *rad_fgetspent(FILE *sphandle) {
 	ptr = buffer;
 	RAD_EXTRACT_FIELD("Username", username);
 	GET_SP_NAME(&spbuf) = username;
-	
+
 	/* Get (encrypted) passwords from the shadow file */
 	if (*ptr != '\0') ptr++;
 	RAD_EXTRACT_FIELD("Password", userpwd);
